/* $Id: main.js 6419 2014-02-07 07:13:18Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson, Gregory Vincic
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	Contains a lot of JavaScript functions that should be available
	on every page.

	@author Nicklas
	@version 2.0
*/

/**
	Get the root URL path of this BASE installation. Always ends with a '/' (eg. /base/)
*/
function getRoot()
{
	return App.getRoot();
}

/**
	Get the session-id of the current session or an empty string if no session has been started yet
	(should only happen on the top frameset of the login page)
*/
function getSessionId()
{
	return App.getSessionId();
}

/**
	Get the scaling factor to use in the gui for popup windows (and other things that may
	require scaling to fit).
*/
function getScale()
{
	return App.getScale();
}

/**
	Get the max length of an URL that is allowed in a GET request. URLs that
	are longer than this should be rewritten to a POST request instead. A
	value of '0' disable the rewriting.
*/
function getMaxUrlLength()
{
	return App.getMaxUrlLength();
}


// Holder for all functions in this script
var Main = new MainClass();
function MainClass()
{
	
	/**
		Add a function that is going to be executed as soon as the page
		has been loaded. This is equivalent to setting window.onload but
		support calling multiple functions. The functations are called 
		in the reverse order as they were added to this method.
	*/
	this.onLoad = function(f)
	{
		Doc.onLoad(f);
	}
	
	/*
		Opens a popup window with the specified name and size and loads the given url
	
		@param url The url to load in the window
		@param name The name of the window
		@param width The width in pixels of the window
		@param height The height in pixels of the window
		@param options Specifies options for the decoration of the window as defined
			by the JavaScript window.open() method
		@param parentWin The window object to use when calling open(). If not given
			the current window is used
		@see http://devedge.netscape.com/library/manuals/2000/javascript/1.3/reference/window.html#1202731
	*/
	this.openPopup = function(url, name, width, height, options, parentWin)
	{
		Dialogs.openPopup(url, name, width, height);
	}

	/**
		Brings a named window to focus.
	*/
	this.focusWindow = function(windowName)
	{
		var win = this.getWindow(windowName);
		if (win) win.focus();
	}
	
	/**
		Get a reference to a named window. If the window
		doesn't exists or if it is closed, a reference to
		the current window is returned.
	 */
	this.getWindow = function(windowName)
	{
		var win = Dialogs.getDialog(windowName);
		if (!win || win.closed) win = window;
		return win;
	}
	
	/*
		Opens a popupwindow with a larger textarea field. The text is taken from
		the specified textarea of the specified form and is saved back to
		when the window is closed.
	
		@param title The title of the window
		@param form The name of the form where the textarea is located
		@param textarea The name of the textarea to zoom
	*/
	this.zoom = function(title, form, textarea, disabled, callback)
	{
		var tmp = document.forms[form][textarea];
		if (!tmp.id) tmp.id = 'z'+(new Date()).getTime();
		Dialogs.openZoom(tmp, title, form == 'help');
	}
	
	/*
		Opens a popup window allowing a user to select a color from a color chart. The 
		current color is taken from the specified input field of the specified form and 
		is saved back to it when the window is closed.
	
		@param title The title of the window
		@param form The name of the form where the input field is located
		@param input The name of the input field that holds the color value
		@param callback If specified, the color is returned by using this javascript call 
			instead of by setting the form field
	*/
	this.selectColor = function(title, form, input, callback)
	{
		var tmp = document.forms[form][input];
		if (!tmp.id) tmp.id = 'z'+(new Date()).getTime();
		Dialogs.openColorChart(tmp, title);
	}
	
	/*
	  	Opens a popupwindow with helptext
	  	
	  	@param session The ID of the session.
	  	@param helpid The id of the helptext to be displayed.
	*/
	this.openHelp = function(ID, helpid)	
	{
		Dialogs.openHelp(helpid);
	}

	/*
	  	Opens a popupwindow with helptext getting the help id from
	  	a tabcontrol.
	  	
	  	@param session The ID of the session
	  	@param tabcontrol The ID of the tabcontrol
	  	@param helpid The id of the helptext to be displayed, only
	  		used if the current tab doesn't specify a help id
	*/
	this.openTabControlHelp = function(ID, tabcontrol, helpId)	
	{
		Dialogs.openTabControlHelp(tabcontrol, helpId);
	}

	
	/**
		Cut a string if it is longer than a specified length. If cut,
		three dots are appended to the string. The returned string is never
		longer than the specified max length.

		@param value The string to cut
		@param maxLength The maximum length
	*/
	this.cutString = function(value, maxLength)
	{
		return Strings.cut(value, maxLength);
	}
	
	/**
		Trim leading and trailing whitespace from a string.
		@param value The string to trim
		@return The trimmed string
	*/
	this.trimString = function(value)
	{
		return Strings.trim(value);
	}
	
	// Show or hide an element with the given ID attribute as a block element
	this.showHide = function(id, show)
	{
		Doc.showHide(id, show);
	}
	
	// Shows an element with the given ID attribute as a block element
	this.show = function(id)
	{
		Doc.show(id);
	}
	// Shows an element with the given ID attribute as an inline element
	this.showInline = function(id)
	{
		Doc.show(id, 'inline');
	}
	// Hides an element with the given ID attribute
	this.hide = function(id)
	{
		Doc.hide(id);
	}
	
	this.addClass = function(obj, className)
	{
		Doc.addClass(obj, className);
	}
	
	this.removeClass = function(obj, className)
	{
		Doc.removeClass(obj, className);
	}
	
	this.addOrRemoveClass = function(obj, className, addIfTrue)
	{
		Doc.addOrRemoveClass(obj, className, addIfTrue);
	}
	
	this.toggleClass = function(obj, className)
	{
		Doc.addOrRemoveClass(obj, className);
	}
	
	// Get the character code of an event that invloves a key, returns 0 if no code can be found
	// Use this method for onKeyPress events only. See also: getEventKeyCode()
	this.getEventCharCode = function(event)
	{
		var charCode = event.charCode;
		if (event.ctrlKey) 
		{
			// CTRL + some key usually doesn't result in a printable character
			charCode = 0;
		}
		if (charCode == undefined)
		{
			// IE8 
			charCode = event.keyCode;
		}
		if (charCode == 0 && event.keyCode < 32)
		{
			// In FF, if event.charCode == 0 a special key was pressed
			// It is safe to return it's keyCode if < 32
			// Also note that IE doesn't send events for all special keys.
			charCode = event.keyCode;
		}
		return charCode;
	}
	
	// Get the key code of an event that invloves a key, returns 0 if no code can be found
	// Use this method for onKeyDown and onKeyUp events only. See also: getEventCharCode();
	this.getEventKeyCode = function(event)
	{
		return event.keyCode;
	}
	
	// Handle keypress events on focused elements:
	//  * Send a 'click' event to the same target if the 'ENTER' key was pressed
	this.handleFocusedKeypress = function(event)
	{
		Events.handleFocusedKeypress(event);
	}
	
	// Try to get the window height
	this.getWindowHeight = function(theWin)
	{
		return App.getWindowPosition(theWin).height;
	}
	
	// Try to get the window width
	this.getWindowWidth = function(theWin)
	{
		return App.getWindowPosition(theWin).width;
	}

	// Get the position and size of an element; 
	// returns an object with 'left', 'top', 'width' and 'height' properties
	// If 'rightAndBottom' is true, then 'right' and 'bottom' properties are also calculated
	this.getElementPosition = function(theElement, rightAndBottom)
	{
		return Doc.getElementPosition(theElement);
	}
	
	
	this.encodeURI = function(value)
	{
		return encodeURIComponent(value);
	}

	this.encodeTags = function(value)
	{
		return Strings.encodeTags(value);
	}
	
	this.viewFile = function(ID, fileId)
	{
		Files.viewFile(fileId);
	}

	this.downloadFile = function(ID, fileId)
	{
		Files.downloadFile(fileId);
	}
	
	
	this.getController = function(itemType)
	{
		return Items.getController(itemType);
	}
	
	this.listItems = function(ID, itemType, extraUrl)
	{
		Items.list(itemType, extraUrl);
	}
		
	this.viewOrEditItem = function(ID, itemType, itemId, edit, extraUrl, popupOptions)
	{
		if (edit)
		{
			Items.editItem(itemType, itemId, extraUrl);
		}
		else
		{
			Items.viewItem(itemType, itemId, extraUrl);
		}
		return;
	}
	
	this.selectItem = function(ID, itemType, mode, callback, extraUrl, popupOptions)
	{
		Dialogs.selectItem(itemType, callback, mode.indexOf('ultiple') > 0, extraUrl);
	}
	
	this.deleteItemPermanently = function(ID, ask, itemType, itemId, extraUrl, popupOptions)
	{
		Items.deleteItemPermanently(itemType, itemId, ask);
	}

	this.showUsingItems = function(ID, itemType, itemId, extraUrl)
	{
		Items.showUsingItems(itemType, itemId);
	}

	/**
		Create a new ajax request.
		@return A request object or null if the browser doesn't support ajax
	*/
	this.getAjaxRequest = function()
	{
		return Ajax.getXmlHttpRequest();
	}


	/*
		Safely set the location of a document using as if
		win.document.location.href = url or win.document.replace(url)
		had been used. If the url is longer
		than specified by getMaxUrlLength() function the url will
		be re-written to a <form> using hidden fields to submit
		query values.
		@param url The url to set
		@param win The window to set the location on, if not given,
			the current window is assumed
		@param replace If given, use location.replace() instead of location.href
			(only if url is not too long)
	*/
	this.safeSetLocation = function(url, win, replace)
	{
		App.safeSetLocation(url, win, replace);
	}
	
	/*
		Check if the URL is longer than the configured max URL length.
	*/
	this.isTooLongURL = function(url)
	{
		return App.isTooLongUrl(url);
	}
	
	/*
		Convert an URL to a HTML form using POST and write it to
		a document. This method	extract all query parameters from the 
		URL (except ID) and	generate a <form> tag with hidden input fields. 
		The method returns an unattached Form object. To post it, 
		do: document.body.appendChild(form); form.submit();
		@param url The url to convert
		@param win The window the form should post to, if no window is given
			or if it doesn't have a name the form is posted to the current
			window.
	*/
	this.convertToPostForm = function(url, win)
	{
		return Forms.convertUrlToForm(url, win);
	}
	
	/**
		Save a key-value pair for a page.
	*/
	this.setPageValue = function(page, key, value)
	{
		Data.setPageValue(key, value, page);
	}

	/**
		Get the saved value for the given key on a page
	*/
	this.getPageValue = function(page, key)
	{
		return Data.getPageValue(key,  page);
	}


	this.debug = function(message, clearOutput)
	{
		App.debug(message);
	}
	
	this.hideDebug = function()
	{}
	
}





